%%%%%%%%%%%1. image i/o
img=imread('dog.jpg');
%you'll get gray in uint8
gray=rgb2gray(img);
imshow(gray);
%however, uint8 may not be the ideal type for applying filters
uint8(255+10)
%it is better to use double
%a double image should have range [0,1] to be displayed properly
gray=rgb2gray(double(img)/255);
imshow(gray);
bw=edge(double(rgb2gray(img))/255,'canny');

%%%%%%%%%%2. display filter results
[xx,yy]=meshgrid(1:300,1:300);
res=10*exp(-((xx-150).^2+(yy-150).^2)/2000);
%you can directly use imagesc to visualize the result
imagesc(res);
%or you can also use imshow, however, the outcome is saturated if you use it directly
imshow(res);
%instead, it is better to normalize the range to [0,1] first
imshow(res/max(res(:)))

%%%%%%%%%%3. separable filters
%let's first construct a separable filter f
u=rand(10,1);
v=rand(1,10);
f=u*v;
%now let's separate it by svd
[a,b,c]=svd(f);
u_sep=abs(a(:,1)*sqrt(b(1,1)));
v_sep=abs(c(:,1)'*sqrt(b(1,1)));
%let's compare v and v_sep
subplot(1,2,1)
plot(u)
hold on
plot(u_sep,'r')
hold off
subplot(1,2,2)
plot(v)
hold on
plot(v_sep,'r')
hold off

%%%%%%%%%%4. image interpolation
%let's use the same dog image, but make it a bit smaller first
img=imresize(rgb2gray(img),[100,200]);
imshow(img);
%now we want to make it larger
[xx,yy]=meshgrid(1:200,1:100);
[xxq,yyq]=meshgrid(1:600,1:300);
newimg1=interp2(xx*3,yy*3,double(img)/255,xxq,yyq);
imshow(newimg1);
%let's try different methods
newimg2=interp2(xx*3,yy*3,double(img)/255,xxq,yyq,'nearest');
imshow(newimg2);
newimg3=interp2(xx*3,yy*3,double(img)/255,xxq,yyq,'cubic');
imshow(newimg3);
%let's compare them
subplot(1,3,1);
imshow(newimg1);
subplot(1,3,2);
imshow(newimg2);
subplot(1,3,3);
imshow(newimg3);